@@ -1,6 +1,3 @@
-# -*- coding: utf-8 -*-
-
-
 __license__   = 'GPL v3'
 __copyright__ = '2009, John Schember <john at nachtimwald.com>'
 __docformat__ = 'restructuredtext en'
@@ -11,27 +8,21 @@ driver. It is intended to be subclassed with the relevant parts implemented
 for a particular device.
 '''
 
-import os, time, json, shutil
+import json
+import os
+import shutil
 from itertools import cycle
 
-from calibre.constants import numeric_version
-from calibre import prints, isbytestring, fsync
-from calibre.constants import filesystem_encoding, DEBUG
+from calibre import fsync, isbytestring, prints
+from calibre.constants import filesystem_encoding, ismacos, numeric_version
+from calibre.devices.usbms.books import Book, BookList
 from calibre.devices.usbms.cli import CLI
 from calibre.devices.usbms.device import Device
-from calibre.devices.usbms.books import BookList, Book
 from calibre.ebooks.metadata.book.json_codec import JsonCodec
+from calibre.prints import debug_print
 from polyglot.builtins import itervalues, string_or_bytes
 
 
-def debug_print(*args, **kw):
-    base_time = getattr(debug_print, 'base_time', None)
-    if base_time is None:
-        debug_print.base_time = base_time = time.monotonic()
-    if DEBUG:
-        prints('DEBUG: %6.1f'%(time.monotonic()-base_time), *args, **kw)
-
-
 def safe_walk(top, topdown=True, onerror=None, followlinks=False, maxdepth=128):
     ' A replacement for os.walk that does not die when it encounters undecodeable filenames in a linux filesystem'
     if maxdepth < 0:
@@ -56,7 +47,7 @@ def safe_walk(top, topdown=True, onerror=None, followlinks=False, maxdepth=128):
             try:
                 name = name.decode(filesystem_encoding)
             except UnicodeDecodeError:
-                debug_print('Skipping undecodeable file: %r' % name)
+                debug_print(f'Skipping undecodeable file: {name!r}')
                 continue
         if isdir(join(top, name)):
             dirs.append(name)
@@ -76,7 +67,6 @@ def safe_walk(top, topdown=True, onerror=None, followlinks=False, maxdepth=128):
 # CLI must come before Device as it implements the CLI functions that
 # are inherited from the device interface in Device.
 class USBMS(CLI, Device):
-
     '''
     The base class for all USBMS devices. Implements the logic for
     sending/getting/updating metadata/caching metadata/etc.
@@ -101,8 +91,9 @@ class USBMS(CLI, Device):
     SCAN_FROM_ROOT = False
 
     def _update_driveinfo_record(self, dinfo, prefix, location_code, name=None):
-        from calibre.utils.date import now, isoformat
         import uuid
+
+        from calibre.utils.date import isoformat, now
         if not isinstance(dinfo, dict):
             dinfo = {}
         if dinfo.get('device_store_uuid', None) is None:
@@ -121,17 +112,17 @@ class USBMS(CLI, Device):
     def _update_driveinfo_file(self, prefix, location_code, name=None):
         from calibre.utils.config import from_json, to_json
         if os.path.exists(os.path.join(prefix, self.DRIVEINFO)):
-            with lopen(os.path.join(prefix, self.DRIVEINFO), 'rb') as f:
+            with open(os.path.join(prefix, self.DRIVEINFO), 'rb') as f:
                 try:
                     driveinfo = json.loads(f.read(), object_hook=from_json)
-                except:
+                except Exception:
                     driveinfo = None
                 driveinfo = self._update_driveinfo_record(driveinfo, prefix,
                                                           location_code, name)
             data = json.dumps(driveinfo, default=to_json)
             if not isinstance(data, bytes):
                 data = data.encode('utf-8')
-            with lopen(os.path.join(prefix, self.DRIVEINFO), 'wb') as f:
+            with open(os.path.join(prefix, self.DRIVEINFO), 'wb') as f:
                 f.write(data)
                 fsync(f)
         else:
@@ -139,7 +130,7 @@ class USBMS(CLI, Device):
             data = json.dumps(driveinfo, default=to_json)
             if not isinstance(data, bytes):
                 data = data.encode('utf-8')
-            with lopen(os.path.join(prefix, self.DRIVEINFO), 'wb') as f:
+            with open(os.path.join(prefix, self.DRIVEINFO), 'wb') as f:
                 f.write(data)
                 fsync(f)
         return driveinfo
@@ -147,17 +138,30 @@ class USBMS(CLI, Device):
     def get_device_information(self, end_session=True):
         self.report_progress(1.0, _('Get device information...'))
         self.driveinfo = {}
+
+        def raise_os_error(e):
+            raise OSError(_('Failed to access files in the main memory of'
+                    ' your device. You should contact the device'
+                    ' manufacturer for support. Common fixes are:'
+                    ' try a different USB cable/USB port on your computer.'
+                    ' If you device has a "Reset to factory defaults" type'
+                    ' of setting somewhere, use it. Underlying error: %s')
+                    % e) from e
+
         if self._main_prefix is not None:
             try:
                 self.driveinfo['main'] = self._update_driveinfo_file(self._main_prefix, 'main')
+            except PermissionError as e:
+                if ismacos:
+                    raise PermissionError(_(
+                        'Permission was denied by macOS trying to access files in the main memory of'
+                        ' your device. You will need to grant permission explicitly by looking under'
+                        ' System Preferences > Security and Privacy > Privacy > Files and Folders.'
+                        ' Underlying error: %s'
+                    ) % e) from e
+                raise_os_error(e)
             except OSError as e:
-                raise OSError(_('Failed to access files in the main memory of'
-                        ' your device. You should contact the device'
-                        ' manufacturer for support. Common fixes are:'
-                        ' try a different USB cable/USB port on your computer.'
-                        ' If you device has a "Reset to factory defaults" type'
-                        ' of setting somewhere, use it. Underlying error: %s')
-                        % e)
+                raise_os_error(e)
         try:
             if self._card_a_prefix is not None:
                 self.driveinfo['A'] = self._update_driveinfo_file(self._card_a_prefix, 'A')
@@ -229,6 +233,9 @@ class USBMS(CLI, Device):
 
         def update_booklist(filename, path, prefix):
             changed = False
+            # Ignore AppleDouble files
+            if filename.startswith('._'):
+                return False
             if path_to_ext(filename) in all_formats and self.is_allowed_book_file(filename, path, prefix):
                 try:
                     lpath = os.path.join(path, filename).partition(self.normalize_path(prefix))[2]
@@ -239,13 +246,13 @@ class USBMS(CLI, Device):
                     if idx is not None:
                         bl_cache[lpath] = None
                         if self.update_metadata_item(bl[idx]):
-                            # print 'update_metadata_item returned true'
+                            # print('update_metadata_item returned true')
                             changed = True
                     else:
                         if bl.add_book(self.book_from_path(prefix, lpath),
                                               replace_metadata=False):
                             changed = True
-                except:  # Probably a filename encoding error
+                except Exception:  # Probably a filename encoding error
                     import traceback
                     traceback.print_exc()
             return changed
@@ -292,8 +299,7 @@ class USBMS(CLI, Device):
                 need_sync = True
                 del bl[idx]
 
-        debug_print('USBMS: count found in cache: %d, count of files in metadata: %d, need_sync: %s' %
-            (len(bl_cache), len(bl), need_sync))
+        debug_print(f'USBMS: count found in cache: {len(bl_cache)}, count of files in metadata: {len(bl)}, need_sync: {need_sync}')
         if need_sync:  # self.count_found_in_bl != len(bl) or need_sync:
             if oncard == 'cardb':
                 self.sync_booklists((None, None, bl))
@@ -308,7 +314,7 @@ class USBMS(CLI, Device):
 
     def upload_books(self, files, names, on_card=None, end_session=True,
                      metadata=None):
-        debug_print('USBMS: uploading %d books'%(len(files)))
+        debug_print(f'USBMS: uploading {len(files)} books')
 
         path = self._sanity_check(on_card, files)
 
@@ -327,14 +333,14 @@ class USBMS(CLI, Device):
                 self.upload_cover(os.path.dirname(filepath),
                                   os.path.splitext(os.path.basename(filepath))[0],
                                   mdata, filepath)
-            except:  # Failure to upload cover is not catastrophic
+            except Exception:  # Failure to upload cover is not catastrophic
                 import traceback
                 traceback.print_exc()
 
             self.report_progress((i+1) / float(len(files)), _('Transferring books to device...'))
 
         self.report_progress(1.0, _('Transferring books to device...'))
-        debug_print('USBMS: finished uploading %d books'%(len(files)))
+        debug_print(f'USBMS: finished uploading {len(files)} books')
         return list(zip(paths, cycle([on_card])))
 
     def upload_cover(self, path, filename, metadata, filepath):
@@ -351,7 +357,7 @@ class USBMS(CLI, Device):
         pass
 
     def add_books_to_metadata(self, locations, metadata, booklists):
-        debug_print('USBMS: adding metadata for %d books'%(len(metadata)))
+        debug_print(f'USBMS: adding metadata for {len(metadata)} books')
 
         metadata = iter(metadata)
         locations = tuple(locations)
@@ -379,7 +385,7 @@ class USBMS(CLI, Device):
                         self._main_prefix)
                 continue
             lpath = path.partition(prefix)[2]
-            if lpath.startswith('/') or lpath.startswith('\\'):
+            if lpath.startswith(('/', '\\')):
                 lpath = lpath[1:]
             book = self.book_class(prefix, lpath, other=info)
             if book.size is None:
@@ -407,11 +413,11 @@ class USBMS(CLI, Device):
         if self.SUPPORTS_SUB_DIRS:
             try:
                 os.removedirs(os.path.dirname(path))
-            except:
+            except Exception:
                 pass
 
     def delete_books(self, paths, end_session=True):
-        debug_print('USBMS: deleting %d books'%(len(paths)))
+        debug_print(f'USBMS: deleting {len(paths)} books')
         for i, path in enumerate(paths):
             self.report_progress((i+1) / float(len(paths)), _('Removing books from device...'))
             path = self.normalize_path(path)
@@ -421,10 +427,10 @@ class USBMS(CLI, Device):
                 self.delete_extra_book_files(path)
 
         self.report_progress(1.0, _('Removing books from device...'))
-        debug_print('USBMS: finished deleting %d books'%(len(paths)))
+        debug_print(f'USBMS: finished deleting {len(paths)} books')
 
     def remove_books_from_metadata(self, paths, booklists):
-        debug_print('USBMS: removing metadata for %d books'%(len(paths)))
+        debug_print(f'USBMS: removing metadata for {len(paths)} books')
 
         for i, path in enumerate(paths):
             self.report_progress((i+1) / float(len(paths)), _('Removing books from device metadata listing...'))
@@ -433,7 +439,7 @@ class USBMS(CLI, Device):
                     if path.endswith(book.path):
                         bl.remove_book(book)
         self.report_progress(1.0, _('Removing books from device metadata listing...'))
-        debug_print('USBMS: finished removing metadata for %d books'%(len(paths)))
+        debug_print(f'USBMS: finished removing metadata for {len(paths)} books')
 
     # If you override this method and you use book._new_book, then you must
     # complete the processing before you call this method. The flag is cleared
@@ -450,7 +456,7 @@ class USBMS(CLI, Device):
                     isinstance(booklists[listid], self.booklist_class)):
                 if not os.path.exists(prefix):
                     os.makedirs(self.normalize_path(prefix))
-                with lopen(self.normalize_path(os.path.join(prefix, self.METADATA_CACHE)), 'wb') as f:
+                with open(self.normalize_path(os.path.join(prefix, self.METADATA_CACHE)), 'wb') as f:
                     json_codec.encode_to_file(f, booklists[listid])
                     fsync(f)
         write_prefix(self._main_prefix, 0)
@@ -496,9 +502,9 @@ class USBMS(CLI, Device):
         cache_file = cls.normalize_path(os.path.join(prefix, name))
         if os.access(cache_file, os.R_OK):
             try:
-                with lopen(cache_file, 'rb') as f:
+                with open(cache_file, 'rb') as f:
                     json_codec.decode_from_file(f, bl, cls.book_class, prefix)
-            except:
+            except Exception:
                 import traceback
                 traceback.print_exc()
                 bl = []
@@ -524,8 +530,8 @@ class USBMS(CLI, Device):
 
     @classmethod
     def metadata_from_formats(cls, fmts):
-        from calibre.ebooks.metadata.meta import metadata_from_formats
         from calibre.customize.ui import quick_metadata
+        from calibre.ebooks.metadata.meta import metadata_from_formats
         with quick_metadata:
             return metadata_from_formats(fmts, force_read_metadata=True,
                                          pattern=cls.build_template_regexp())
