@@ -98,8 +98,8 @@ def finalize(shortcuts, custom_keys_map={}):  # {{{
             x = str(ks.toString(QKeySequence.SequenceFormat.PortableText))
             if x in seen:
                 if DEBUG:
-                    prints('Key %r for shortcut %s is already used by'
-                            ' %s, ignoring'%(x, shortcut['name'], seen[x]['name']))
+                    prints('Key {!r} for shortcut {} is already used by'
+                            ' {}, ignoring'.format(x, shortcut['name'], seen[x]['name']))
                 keys_map[unique_name] = ()
                 continue
             seen[x] = shortcut
@@ -110,12 +110,11 @@ def finalize(shortcuts, custom_keys_map={}):  # {{{
         ac = shortcut['action']
         if ac is None or sip.isdeleted(ac):
             if ac is not None and DEBUG:
-                prints('Shortcut %r has a deleted action' % unique_name)
+                prints(f'Shortcut {unique_name!r} has a deleted action')
             continue
         ac.setShortcuts(list(keys))
 
     return keys_map
-
 # }}}
 
 
@@ -156,8 +155,7 @@ class Manager(QObject):  # {{{
         '''
         if unique_name in self.shortcuts:
             name = self.shortcuts[unique_name]['name']
-            raise NameConflict('Shortcut for %r already registered by %s'%(
-                    unique_name, name))
+            raise NameConflict(f'Shortcut for {unique_name!r} already registered by {name}')
         shortcut = {'name':name, 'desc':description, 'action': action,
                 'default_keys':tuple(default_keys),
                 'persist_shortcut':persist_shortcut}
@@ -194,8 +192,8 @@ class Manager(QObject):  # {{{
 
 # }}}
 
-# Model {{{
 
+# Model {{{
 
 class Node:
 
@@ -259,7 +257,7 @@ class ConfigModel(SearchQueryParser, QAbstractItemModel):
             ip = self.data
         try:
             return self.createIndex(row, column, ip[row])
-        except:
+        except Exception:
             pass
         return ROOT
 
@@ -318,10 +316,8 @@ class ConfigModel(SearchQueryParser, QAbstractItemModel):
             sc = node.data
             un = sc['unique_name']
             if sc['set_to_default']:
-                if un in kmap:
-                    del kmap[un]
-                if un in options_map:
-                    del options_map[un]
+                kmap.pop(un, None)
+                options_map.pop(un, None)
             else:
                 if sc['persist_shortcut']:
                     options_map[un] = options_map.get(un, {})
@@ -354,7 +350,7 @@ class ConfigModel(SearchQueryParser, QAbstractItemModel):
             else:
                 try:
                     sc = self.data[c].children[p].data
-                except:
+                except Exception:
                     continue
                 if query in lower(sc['name']):
                     ans.add((c, p))
@@ -446,29 +442,29 @@ class Editor(QFrame):  # {{{
             la = QLabel(text)
             la.setStyleSheet('QLabel { margin-left: 1.5em }')
             l.addWidget(la, off+which, 0, 1, 3)
-            setattr(self, 'label%d'%which, la)
+            setattr(self, f'label{which}', la)
             button = QPushButton(_('None'), self)
             button.setObjectName(_('None'))
             button.clicked.connect(partial(self.capture_clicked, which=which))
             button.installEventFilter(self)
-            setattr(self, 'button%d'%which, button)
+            setattr(self, f'button{which}', button)
             clear = QToolButton(self)
             clear.setIcon(QIcon.ic('clear_left.png'))
             clear.clicked.connect(partial(self.clear_clicked, which=which))
-            setattr(self, 'clear%d'%which, clear)
+            setattr(self, f'clear{which}', clear)
             l.addWidget(button, off+which, 1, 1, 1)
             l.addWidget(clear, off+which, 2, 1, 1)
             la.setBuddy(button)
 
         self.done_button = doneb = QPushButton(_('Done'), self)
         l.addWidget(doneb, 0, 2, 1, 1)
-        doneb.clicked.connect(lambda : self.editing_done.emit(self))
+        doneb.clicked.connect(lambda: self.editing_done.emit(self))
         l.setColumnStretch(0, 100)
 
         self.custom_toggled(False)
 
     def initialize(self, shortcut, all_shortcuts):
-        self.header.setText('<b>%s: %s</b>'%(_('Customize'), shortcut['name']))
+        self.header.setText('<b>{}: {}</b>'.format(_('Customize'), shortcut['name']))
         self.all_shortcuts = all_shortcuts
         self.shortcut = shortcut
 
@@ -492,7 +488,7 @@ class Editor(QFrame):  # {{{
         else:
             self.use_custom.setChecked(True)
             for key, which in zip(self.current_keys, [1,2]):
-                button = getattr(self, 'button%d'%which)
+                button = getattr(self, f'button{which}')
                 ns = key.toString(QKeySequence.SequenceFormat.NativeText)
                 button.setText(ns.replace('&', '&&'))
                 button.setObjectName(ns)
@@ -504,13 +500,13 @@ class Editor(QFrame):  # {{{
 
     def capture_clicked(self, which=1):
         self.capture = which
-        button = getattr(self, 'button%d'%which)
+        button = getattr(self, f'button{which}')
         button.setText(_('Press a key...'))
         button.setFocus(Qt.FocusReason.OtherFocusReason)
         button.setStyleSheet('QPushButton { font-weight: bold}')
 
     def clear_clicked(self, which=0):
-        button = getattr(self, 'button%d'%which)
+        button = getattr(self, f'button{which}')
         button.setText(_('None'))
         button.setObjectName(_('None'))
 
@@ -533,7 +529,7 @@ class Editor(QFrame):  # {{{
             return QWidget.keyPressEvent(self, ev)
         ev.accept()
 
-        button = getattr(self, 'button%d'%which)
+        button = getattr(self, f'button{which}')
         button.setStyleSheet('QPushButton { font-weight: normal}')
         ns = sequence.toString(QKeySequence.SequenceFormat.NativeText)
         button.setText(ns.replace('&', '&&'))
@@ -560,7 +556,7 @@ class Editor(QFrame):  # {{{
             return None
         ans = []
         for which in (1, 2):
-            button = getattr(self, 'button%d'%which)
+            button = getattr(self, f'button{which}')
             t = button.objectName()
             if not t or t == _('None'):
                 continue
@@ -569,9 +565,9 @@ class Editor(QFrame):  # {{{
                 ans.append(ks)
         return tuple(ans)
 
-
 # }}}
 
+
 class Delegate(QStyledItemDelegate):  # {{{
 
     changed_signal = pyqtSignal()
@@ -593,12 +589,12 @@ class Delegate(QStyledItemDelegate):  # {{{
                 keys = _('None')
             else:
                 keys = ', '.join(keys)
-            html = '<b>%s</b><br>%s: %s'%(
+            html = '<b>{}</b><br>{}: {}'.format(
                 prepare_string_for_xml(shortcut['name']), _('Shortcuts'), prepare_string_for_xml(keys))
         else:
             # Group
             html = data.data
-        doc =  QTextDocument()
+        doc = QTextDocument()
         doc.setHtml(html)
         return doc
 
