@@ -1,5 +1,4 @@
 #!/usr/bin/env python
-# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 
 
 __license__   = 'GPL v3'
@@ -9,36 +8,70 @@ __docformat__ = 'restructuredtext en'
 DEBUG_DIALOG = False
 
 # Imports {{{
-import os, time
-from threading import Thread, Event
-from operator import attrgetter
+import os
+import time
 from io import BytesIO
+from operator import attrgetter
+from threading import Event, Thread
 
 from qt.core import (
-    QStyledItemDelegate, QTextDocument, QRectF, QIcon, Qt, QApplication,
-    QDialog, QVBoxLayout, QLabel, QDialogButtonBox, QStyle, QStackedWidget,
-    QWidget, QTableView, QGridLayout, QPalette, QTimer, pyqtSignal,
-    QAbstractTableModel, QSize, QListView, QPixmap, QModelIndex,
-    QAbstractListModel, QRect, QTextBrowser, QStringListModel, QMenu, QItemSelectionModel,
-    QCursor, QHBoxLayout, QPushButton, QSizePolicy, QSplitter, QAbstractItemView)
+    QAbstractItemView,
+    QAbstractListModel,
+    QAbstractTableModel,
+    QApplication,
+    QCursor,
+    QDialog,
+    QDialogButtonBox,
+    QGridLayout,
+    QHBoxLayout,
+    QIcon,
+    QItemSelectionModel,
+    QLabel,
+    QListView,
+    QMenu,
+    QModelIndex,
+    QPalette,
+    QPixmap,
+    QPushButton,
+    QRect,
+    QRectF,
+    QSize,
+    QSizePolicy,
+    QSplitter,
+    QStackedWidget,
+    QStringListModel,
+    QStyle,
+    QStyledItemDelegate,
+    Qt,
+    QTableView,
+    QTextBrowser,
+    QTextDocument,
+    QTimer,
+    QVBoxLayout,
+    QWidget,
+    pyqtSignal,
+)
 
+from calibre import force_unicode
 from calibre.customize.ui import metadata_plugins
+from calibre.db.constants import COVER_FILE_NAME, DATA_DIR_NAME
 from calibre.ebooks.metadata import authors_to_string, rating_to_stars
-from calibre.utils.logging import GUILog as Log
-from calibre.ebooks.metadata.sources.identify import urls_from_identifiers
 from calibre.ebooks.metadata.book.base import Metadata
 from calibre.ebooks.metadata.opf2 import OPF
-from calibre.gui2 import error_dialog, rating_font, gprefs
+from calibre.ebooks.metadata.sources.identify import urls_from_identifiers
+from calibre.gui2 import choose_save_file, error_dialog, gprefs, rating_font
 from calibre.gui2.progress_indicator import SpinAnimator
 from calibre.gui2.widgets2 import HTMLDisplay
-from calibre.utils.date import (utcnow, fromordinal, format_date,
-        UNDEFINED_DATE, as_utc)
 from calibre.library.comments import comments_to_html
-from calibre import force_unicode
-from calibre.utils.ipc.simple_worker import fork_job, WorkerError
 from calibre.ptempfile import TemporaryDirectory
+from calibre.utils.date import UNDEFINED_DATE, as_utc, format_date, fromordinal, utcnow
+from calibre.utils.img import image_to_data, save_image
+from calibre.utils.ipc.simple_worker import WorkerError, fork_job
+from calibre.utils.logging import GUILog as Log
+from calibre.utils.resources import get_image_path as I
 from polyglot.builtins import iteritems, itervalues
-from polyglot.queue import Queue, Empty
+from polyglot.queue import Empty, Queue
+
 # }}}
 
 
@@ -57,8 +90,8 @@ class RichTextDelegate(QStyledItemDelegate):  # {{{
             group = (QPalette.ColorGroup.Active if option.state & QStyle.StateFlag.State_Active else
                     QPalette.ColorGroup.Inactive)
             c = p.color(group, QPalette.ColorRole.HighlightedText)
-            c = 'rgb(%d, %d, %d)'%c.getRgb()[:3]
-            doc.setDefaultStyleSheet(' * { color: %s }'%c)
+            c = 'rgb({}, {}, {})'.format(*c.getRgb()[:3])
+            doc.setDefaultStyleSheet(f' * {{ color: {c} }}')
         doc.setHtml(index.data() or '')
         return doc
 
@@ -126,7 +159,7 @@ class ResultsModel(QAbstractTableModel):  # {{{
     def __init__(self, results, parent=None):
         QAbstractTableModel.__init__(self, parent)
         self.results = results
-        self.yes_icon = (QIcon(I('ok.png')))
+        self.yes_icon = (QIcon.ic('ok.png'))
 
     def rowCount(self, parent=None):
         return len(self.results)
@@ -138,7 +171,7 @@ class ResultsModel(QAbstractTableModel):  # {{{
         if orientation == Qt.Orientation.Horizontal and role == Qt.ItemDataRole.DisplayRole:
             try:
                 return (self.COLUMNS[section])
-            except:
+            except Exception:
                 return None
         return None
 
@@ -148,17 +181,17 @@ class ResultsModel(QAbstractTableModel):  # {{{
         if col == 1:
             t = book.title if book.title else _('Unknown')
             a = authors_to_string(book.authors) if book.authors else ''
-            return '<b>%s</b><br><i>%s</i>' % (t, a)
+            return f'<b>{t}</b><br><i>{a}</i>'
         if col == 2:
             d = format_date(book.pubdate, 'yyyy') if book.pubdate else _('Unknown')
             p = book.publisher if book.publisher else ''
-            return '<b>%s</b><br><i>%s</i>' % (d, p)
+            return f'<b>{d}</b><br><i>{p}</i>'
 
     def data(self, index, role):
         row, col = index.row(), index.column()
         try:
             book = self.results[row]
-        except:
+        except Exception:
             return None
         if role == Qt.ItemDataRole.DisplayRole and col not in self.ICON_COLS:
             res = self.data_as_text(book, col)
@@ -182,7 +215,6 @@ class ResultsModel(QAbstractTableModel):  # {{{
         return None
 
     def sort(self, col, order=Qt.SortOrder.AscendingOrder):
-        key = lambda x: x
         if col == 0:
             key = attrgetter('gui_rank')
         elif col == 1:
@@ -197,7 +229,11 @@ class ResultsModel(QAbstractTableModel):  # {{{
         elif col == 3:
             key = attrgetter('has_cached_cover_url')
         elif key == 4:
-            key = lambda x: bool(x.comments)
+            def key(x):
+                return bool(x.comments)
+        else:
+            def key(x):
+                return x
 
         self.beginResetModel()
         self.results.sort(key=key, reverse=order==Qt.SortOrder.AscendingOrder)
@@ -255,24 +291,24 @@ class ResultsView(QTableView):  # {{{
         book = self.model().data(index, Qt.ItemDataRole.UserRole)
         parts = [
             '<center>',
-            '<h2>%s</h2>'%book.title,
-            '<div><i>%s</i></div>'%authors_to_string(book.authors),
+            f'<h2>{book.title}</h2>',
+            f'<div><i>{authors_to_string(book.authors)}</i></div>',
         ]
         if not book.is_null('series'):
             series = book.format_field('series')
             if series[1]:
-                parts.append('<div>%s: %s</div>'%series)
+                parts.append('<div>{}: {}</div>'.format(*series))
         if not book.is_null('rating'):
-            style = 'style=\'font-family:"%s"\''%f
-            parts.append('<div %s>%s</div>'%(style, rating_to_stars(int(2 * book.rating))))
+            style = f'style=\'font-family:"{f}"\''
+            parts.append(f'<div {style}>{rating_to_stars(int(2 * book.rating))}</div>')
         parts.append('</center>')
         if book.identifiers:
             urls = urls_from_identifiers(book.identifiers)
-            ids = ['<a href="%s">%s</a>'%(url, name) for name, ign, ign, url in urls]
+            ids = [f'<a href="{url}">{name}</a>' for name, ign, ign, url in urls]
             if ids:
-                parts.append('<div><b>%s:</b> %s</div><br>'%(_('See at'), ', '.join(ids)))
+                parts.append('<div><b>{}:</b> {}</div><br>'.format(_('See at'), ', '.join(ids)))
         if book.tags:
-            parts.append('<div>%s</div><div>\u00a0</div>'%', '.join(book.tags))
+            parts.append('<div>{}</div><div>\u00a0</div>'.format(', '.join(book.tags)))
         if book.comments:
             parts.append(comments_to_html(book.comments))
 
@@ -346,23 +382,23 @@ class Comments(HTMLDisplay):  # {{{
 
         c = color_to_string(QApplication.palette().color(QPalette.ColorGroup.Normal,
                         QPalette.ColorRole.WindowText))
-        templ = '''\
+        templ = f'''\
         <html>
             <head>
             <style type="text/css">
-                body, td {background-color: transparent; color: %s }
-                a { text-decoration: none; }
-                div.description { margin-top: 0; padding-top: 0; text-indent: 0 }
-                table { margin-bottom: 0; padding-bottom: 0; }
+                body, td {{background-color: transparent; color: {c} }}
+                a {{ text-decoration: none; }}
+                div.description {{ margin-top: 0; padding-top: 0; text-indent: 0 }}
+                table {{ margin-bottom: 0; padding-bottom: 0; }}
             </style>
             </head>
             <body>
             <div class="description">
-            %%s
+            %s
             </div>
             </body>
         <html>
-        '''%(c,)
+        '''
         self.setHtml(templ%html)
 # }}}
 
@@ -387,7 +423,7 @@ class IdentifyWorker(Thread):  # {{{
         m1.has_cached_cover_url = True
         m2.has_cached_cover_url = False
         m1.comments  = 'Some comments '*10
-        m1.tags = ['tag%d'%i for i in range(20)]
+        m1.tags = [f'tag{i}' for i in range(20)]
         m1.rating = 4.4
         m1.language = 'en'
         m2.language = 'fr'
@@ -418,7 +454,7 @@ class IdentifyWorker(Thread):  # {{{
                 result.gui_rank = i
         except WorkerError as e:
             self.error = force_unicode(e.orig_tb)
-        except:
+        except Exception:
             import traceback
             self.error = force_unicode(traceback.format_exc())
 
@@ -463,6 +499,7 @@ class IdentifyWidget(QWidget):  # {{{
 
         self.query = QLabel('download starting...')
         self.query.setWordWrap(True)
+        self.query.setTextFormat(Qt.TextFormat.PlainText)
         l.addWidget(self.query)
 
         self.comments_view.show_wait()
@@ -487,10 +524,10 @@ class IdentifyWidget(QWidget):  # {{{
             parts.append('authors:'+authors_to_string(authors))
             simple_desc += _('Authors: %s ') % authors_to_string(authors)
         if identifiers:
-            x = ', '.join('%s:%s'%(k, v) for k, v in iteritems(identifiers))
+            x = ', '.join(f'{k}:{v}' for k, v in iteritems(identifiers))
             parts.append(x)
             if 'isbn' in identifiers:
-                simple_desc += 'ISBN: %s' % identifiers['isbn']
+                simple_desc += 'ISBN: {}'.format(identifiers['isbn'])
         self.query.setText(simple_desc)
         self.log(str(self.query.text()))
 
@@ -521,7 +558,7 @@ class IdentifyWidget(QWidget):  # {{{
             error_dialog(self, _('No matches found'), '<p>' +
                     _('Failed to find any books that '
                         'match your search. Try making the search <b>less '
-                        'specific</b>. For example, use only the author\'s '
+                        "specific</b>. For example, use only the author's "
                         'last name and a single distinctive word from '
                         'the title.<p>To see the full log, click "Show details".'),
                     show=True, det_msg=log)
@@ -564,7 +601,7 @@ class CoverWorker(Thread):  # {{{
                 self.run_fork()
         except WorkerError as e:
             self.error = force_unicode(e.orig_tb)
-        except:
+        except Exception:
             import traceback
             self.error = force_unicode(traceback.format_exc())
 
@@ -598,7 +635,7 @@ class CoverWorker(Thread):  # {{{
                     width, height = int(width), int(height)
                     with open(os.path.join(tdir, x), 'rb') as f:
                         data = f.read()
-                except:
+                except Exception:
                     import traceback
                     traceback.print_exc()
                 else:
@@ -622,10 +659,11 @@ class CoversModel(QAbstractListModel):  # {{{
         QAbstractListModel.__init__(self, parent)
 
         if current_cover is None:
-            current_cover = QPixmap(I('default_cover.png'))
+            ic = QIcon.ic('default_cover.png')
+            current_cover = ic.pixmap(ic.availableSizes()[0])
         current_cover.setDevicePixelRatio(QApplication.instance().devicePixelRatio())
 
-        self.blank = QIcon(I('blank.png')).pixmap(*CoverDelegate.ICON_SIZE)
+        self.blank = QIcon.ic('blank.png').pixmap(*CoverDelegate.ICON_SIZE)
         self.cc = current_cover
         self.reset_covers(do_reset=False)
 
@@ -641,7 +679,7 @@ class CoversModel(QAbstractListModel):  # {{{
             self.beginResetModel(), self.endResetModel()
 
     def get_item(self, src, pmap, waiting=False):
-        sz = '%dx%d'%(pmap.width(), pmap.height())
+        sz = f'{pmap.width()}x{pmap.height()}'
         text = (src + '\n' + sz)
         scaled = pmap.scaled(
             int(CoverDelegate.ICON_SIZE[0] * pmap.devicePixelRatio()), int(CoverDelegate.ICON_SIZE[1] * pmap.devicePixelRatio()),
@@ -655,7 +693,7 @@ class CoversModel(QAbstractListModel):  # {{{
     def data(self, index, role):
         try:
             text, pmap, cover, waiting = self.covers[index.row()]
-        except:
+        except Exception:
             return None
         if role == Qt.ItemDataRole.DecorationRole:
             return pmap
@@ -766,6 +804,7 @@ class CoversView(QListView):  # {{{
 
     def __init__(self, current_cover, parent=None):
         QListView.__init__(self, parent)
+        self.book_id = getattr(parent, 'book_id', 0)
         self.m = CoversModel(current_cover, self)
         self.setModel(self.m)
 
@@ -820,28 +859,60 @@ class CoversView(QListView):  # {{{
         idx = self.currentIndex()
         if idx and idx.isValid() and not idx.data(Qt.ItemDataRole.UserRole):
             m = QMenu(self)
-            m.addAction(QIcon(I('view.png')), _('View this cover at full size'), self.show_cover)
-            m.addAction(QIcon(I('edit-copy.png')), _('Copy this cover to clipboard'), self.copy_cover)
-            m.exec_(QCursor.pos())
+            m.addAction(QIcon.ic('view.png'), _('View this cover at full size'), self.show_cover)
+            m.addAction(QIcon.ic('edit-copy.png'), _('Copy this cover to clipboard'), self.copy_cover)
+            m.addAction(QIcon.ic('save.png'), _('Save this cover to disk'), self.save_to_disk)
+            if self.book_id:
+                m.addAction(QIcon.ic('save.png'), _('Save this cover in the book extra files'), self.save_alternate_cover)
+            m.exec(QCursor.pos())
 
-    def show_cover(self):
+    @property
+    def current_pixmap(self):
         idx = self.currentIndex()
         pmap = self.model().cover_pixmap(idx)
         if pmap is None and idx.row() == 0:
             pmap = self.model().cc
+        return pmap
+
+    def show_cover(self):
+        pmap = self.current_pixmap
         if pmap is not None:
             from calibre.gui2.image_popup import ImageView
-            d = ImageView(self, pmap, str(idx.data(Qt.ItemDataRole.DisplayRole) or ''), geom_name='metadata_download_cover_popup_geom')
+            d = ImageView(self, pmap, str(self.currentIndex().data(Qt.ItemDataRole.DisplayRole) or ''), geom_name='metadata_download_cover_popup_geom')
             d(use_exec=True)
 
     def copy_cover(self):
-        idx = self.currentIndex()
-        pmap = self.model().cover_pixmap(idx)
-        if pmap is None and idx.row() == 0:
-            pmap = self.model().cc
+        pmap = self.current_pixmap
         if pmap is not None:
             QApplication.clipboard().setPixmap(pmap)
 
+    def save_to_disk(self):
+        pmap = self.current_pixmap
+        if pmap:
+            path = choose_save_file(self, 'save-downloaded-cover-to-disk', _('Save cover image'), filters=[
+                (_('Images'), ['jpg', 'jpeg', 'png', 'webp'])], all_files=False, initial_filename=COVER_FILE_NAME)
+            if path:
+                save_image(pmap.toImage(), path)
+
+    def save_alternate_cover(self):
+        pmap = self.current_pixmap
+        if pmap:
+            from calibre.gui2.ui import get_gui
+            db = get_gui().current_db.new_api
+            existing = {x[0] for x in db.list_extra_files(self.book_id)}
+            h, ext = os.path.splitext(COVER_FILE_NAME)
+            template = f'{DATA_DIR_NAME}/{h}-{{:03d}}{ext}'
+            for i in range(1, 1000):
+                q = template.format(i)
+                if q not in existing:
+                    cdata = image_to_data(pmap.toImage())
+                    db.add_extra_files(self.book_id, {q: BytesIO(cdata)}, replace=False, auto_rename=True)
+                    break
+            else:
+                error_dialog(self, _('Too many covers'), _(
+                    'Could not save cover as there are too many existing covers'), show=True)
+                return
+
     def keyPressEvent(self, ev):
         if ev.key() in (Qt.Key.Key_Enter, Qt.Key.Key_Return):
             self.chosen.emit()
@@ -867,6 +938,7 @@ class CoversWidget(QWidget):  # {{{
 
         self.msg = QLabel()
         self.msg.setWordWrap(True)
+        self.book_id = getattr(parent, 'book_id', 0)
         l.addWidget(self.msg, 0, 0)
 
         self.covers_view = CoversView(current_cover, self)
@@ -980,12 +1052,12 @@ class LogViewer(QDialog):  # {{{
         self.copy_button = self.bb.addButton(_('Copy to clipboard'),
                 QDialogButtonBox.ButtonRole.ActionRole)
         self.copy_button.clicked.connect(self.copy_to_clipboard)
-        self.copy_button.setIcon(QIcon(I('edit-copy.png')))
+        self.copy_button.setIcon(QIcon.ic('edit-copy.png'))
         self.bb.rejected.connect(self.reject)
         self.bb.accepted.connect(self.accept)
 
         self.setWindowTitle(_('Download log'))
-        self.setWindowIcon(QIcon(I('debug.png')))
+        self.setWindowIcon(QIcon.ic('debug.png'))
         self.resize(QSize(800, 400))
 
         self.keep_updating = True
@@ -1007,7 +1079,7 @@ class LogViewer(QDialog):  # {{{
         html = self.log.html
         if html != self.last_html:
             self.last_html = html
-            self.tb.setHtml('<pre style="font-family:monospace">%s</pre>'%html)
+            self.tb.setHtml(f'<pre style="font-family:monospace">{html}</pre>')
         QTimer.singleShot(1000, self.update_log)
 
 # }}}
@@ -1018,11 +1090,12 @@ class FullFetch(QDialog):  # {{{
     def __init__(self, current_cover=None, parent=None):
         QDialog.__init__(self, parent)
         self.current_cover = current_cover
+        self.book_id = getattr(parent, 'book_id', 0)
         self.log = Log()
         self.book = self.cover_pixmap = None
 
         self.setWindowTitle(_('Downloading metadata...'))
-        self.setWindowIcon(QIcon(I('download-metadata.png')))
+        self.setWindowIcon(QIcon.ic('download-metadata.png'))
 
         self.stack = QStackedWidget()
         self.l = l = QVBoxLayout()
@@ -1037,12 +1110,12 @@ class FullFetch(QDialog):  # {{{
         self.ok_button = self.bb.button(QDialogButtonBox.StandardButton.Ok)
         self.ok_button.setEnabled(False)
         self.ok_button.clicked.connect(self.ok_clicked)
-        self.prev_button = pb = QPushButton(QIcon(I('back.png')), _('&Back'), self)
+        self.prev_button = pb = QPushButton(QIcon.ic('back.png'), _('&Back'), self)
         pb.clicked.connect(self.back_clicked)
         pb.setSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)
         self.log_button = self.bb.addButton(_('&View log'), QDialogButtonBox.ButtonRole.ActionRole)
         self.log_button.clicked.connect(self.view_log)
-        self.log_button.setIcon(QIcon(I('debug.png')))
+        self.log_button.setIcon(QIcon.ic('debug.png'))
         self.prev_button.setVisible(False)
         h.addWidget(self.prev_button), h.addWidget(self.bb)
 
@@ -1056,11 +1129,8 @@ class FullFetch(QDialog):  # {{{
         self.covers_widget.chosen.connect(self.ok_clicked)
         self.stack.addWidget(self.covers_widget)
 
-        self.resize(850, 600)
-        geom = gprefs.get('metadata_single_gui_geom', None)
-        if geom is not None and geom:
-            QApplication.instance().safe_restore_geometry(self, geom)
-
+        if not self.restore_geometry(gprefs, 'metadata_single_gui_geom'):
+            self.resize(850, 600)
         self.finished.connect(self.cleanup)
 
     def view_log(self):
@@ -1083,7 +1153,7 @@ class FullFetch(QDialog):  # {{{
 
     def accept(self):
         # Prevent the usual dialog accept mechanisms from working
-        gprefs['metadata_single_gui_geom'] = bytearray(self.saveGeometry())
+        self.save_geometry(gprefs, 'metadata_single_gui_geom')
         self.identify_widget.save_state()
         if DEBUG_DIALOG:
             if self.stack.currentIndex() == 2:
@@ -1093,7 +1163,7 @@ class FullFetch(QDialog):  # {{{
                 return QDialog.accept(self)
 
     def reject(self):
-        gprefs['metadata_single_gui_geom'] = bytearray(self.saveGeometry())
+        self.save_geometry(gprefs, 'metadata_single_gui_geom')
         self.identify_widget.cancel()
         self.covers_widget.cancel()
         return QDialog.reject(self)
@@ -1105,7 +1175,7 @@ class FullFetch(QDialog):  # {{{
         self.ok_button.setEnabled(True)
 
     def next_clicked(self, *args):
-        gprefs['metadata_single_gui_geom'] = bytearray(self.saveGeometry())
+        self.save_geometry(gprefs, 'metadata_single_gui_geom')
         self.identify_widget.get_result()
 
     def ok_clicked(self, *args):
@@ -1126,7 +1196,7 @@ class FullFetch(QDialog):  # {{{
         self.title, self.authors = title, authors
         self.identify_widget.start(title=title, authors=authors,
                 identifiers=identifiers)
-        return self.exec_()
+        return self.exec()
 # }}}
 
 
@@ -1135,11 +1205,12 @@ class CoverFetch(QDialog):  # {{{
     def __init__(self, current_cover=None, parent=None):
         QDialog.__init__(self, parent)
         self.current_cover = current_cover
+        self.book_id = getattr(parent, 'book_id', 0)
         self.log = Log()
         self.cover_pixmap = None
 
         self.setWindowTitle(_('Downloading cover...'))
-        self.setWindowIcon(QIcon(I('default_cover.png')))
+        self.setWindowIcon(QIcon.ic('default_cover.png'))
 
         self.l = l = QVBoxLayout()
         self.setLayout(l)
@@ -1156,24 +1227,21 @@ class CoverFetch(QDialog):  # {{{
         l.addWidget(self.bb)
         self.log_button = self.bb.addButton(_('&View log'), QDialogButtonBox.ButtonRole.ActionRole)
         self.log_button.clicked.connect(self.view_log)
-        self.log_button.setIcon(QIcon(I('debug.png')))
+        self.log_button.setIcon(QIcon.ic('debug.png'))
         self.bb.rejected.connect(self.reject)
         self.bb.accepted.connect(self.accept)
-
-        geom = gprefs.get('single-cover-fetch-dialog-geometry', None)
-        if geom is not None:
-            QApplication.instance().safe_restore_geometry(self, geom)
+        self.restore_geometry(gprefs, 'single-cover-fetch-dialog-geometry')
 
     def cleanup(self):
         self.covers_widget.cleanup()
 
     def reject(self):
-        gprefs.set('single-cover-fetch-dialog-geometry', bytearray(self.saveGeometry()))
+        self.save_geometry(gprefs, 'single-cover-fetch-dialog-geometry')
         self.covers_widget.cancel()
         return QDialog.reject(self)
 
     def accept(self, *args):
-        gprefs.set('single-cover-fetch-dialog-geometry', bytearray(self.saveGeometry()))
+        self.save_geometry(gprefs, 'single-cover-fetch-dialog-geometry')
         self.cover_pixmap = self.covers_widget.cover_pixmap()
         QDialog.accept(self)
 
@@ -1182,7 +1250,7 @@ class CoverFetch(QDialog):  # {{{
         book.identifiers = identifiers
         self.covers_widget.start(book, self.current_cover,
                 title, authors, {})
-        return self.exec_()
+        return self.exec()
 
     def view_log(self):
         self._lv = LogViewer(self.log, self)
